# -*- coding: cp1252 -*-
'''
GTKRawGallery DropBox uploader;
copyright  2013 Daniele Isca
license: GNU GPL v.3 (see COPYING.txt);
This software comes with no warranty of any kind, use at your own risk!
'''
import gtk
import pygtk
import os.path
import os
import traceback
import sys
import time
from dbupload import DropboxConnection


class Publisher:
    def __init__(self, gallery, image_list, gtkrg_datadir):
        self.gallery = gallery
        self.image_list = image_list
        self.stop = False
        self.uploading = False
        wtree = gtk.Builder()
        wtree.add_from_file(os.path.join(gtkrg_datadir, 'publisher.ui'))
        self.initialize_widgets(wtree)
        self.boxmodel1 = gtk.ListStore(str)
        cell = gtk.CellRendererText()
        self.combobox1.set_model(self.boxmodel1)
        self.combobox1.pack_start(cell, True)
        self.combobox1.add_attribute(cell, 'text', 0)
        self.window1.set_icon_from_file(self.gallery.get_icon_path('dropbox.png'))
        self.dialog1.set_icon_from_file(self.gallery.get_icon_path('dropbox.png'))
        self.make_dp_treeview()
        self.entry3.set_activates_default(True)
        self.dialog1.set_default_response(-5)
        self.connect_signals()
        self.window1.set_title(_('Upload into DropBox'))
        self.dialog1.set_title(_('Log into DropBox'))
        if self.gallery.settings.has_key('dropbox_user'):
            self.entry2.set_text(self.gallery.settings['dropbox_user'])
        if self.gallery.settings.has_key('dropbox_pwd'):
            self.entry3.set_text(self.gallery.settings['dropbox_pwd'])
        self.checkbutton1.set_property('visible', False)
        self.checkbutton2.set_property('visible', False)
        self.checkbutton3.set_property('visible', False)
        self.combobox2.set_property('visible', False)
        self.label5.set_property('visible', False)
        self.label4.set_property('visible', False)
        self.radiobutton2.set_property('visible', False)
        self.entry1.set_property('visible', False)
        self.entry4.set_property('visible', False)
        self.radiobutton1.destroy()
        label = gtk.Label('Upload to')
        self.table1.attach(label,0,1,0,1,0,0,5,0)
        label.show()

    def make_dp_treeview(self):
        self.model = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
        renderer_pix = gtk.CellRendererPixbuf()
        column_pix = gtk.TreeViewColumn("Preview", renderer_pix, pixbuf=0)
        column_pix.set_cell_data_func (renderer_pix, self.cell_data_function, None)
        self.treeview1.append_column(column_pix)
        cell = gtk.CellRendererText()                                                                           
        column_item1 = gtk.TreeViewColumn("Filename", cell, text=1)
        column_item1.set_cell_data_func (cell, self.cell_data_function, None)
        column_item1.set_resizable(True)                                                                                         
        self.treeview1.append_column(column_item1)
        cell = gtk.CellRendererText()
        column_status = gtk.TreeViewColumn("Status", cell, text=2)
        column_status.set_cell_data_func (cell, self.cell_data_function, None)
        self.treeview1.append_column(column_status)
        self.treeview1.set_model(self.model)

    def cell_data_function(self, column, cell, model, _iter, data=None):
        value = model.get_value(_iter, 2)
        if value == None:
            color = self.treeview1.rc_get_style().base[gtk.STATE_NORMAL].to_string()
        elif value == 'uploading...':
            color = '#CD7300'
        elif value == 'error!':
            color = 'red'
        else:
            color = self.treeview1.rc_get_style().base[gtk.STATE_NORMAL].to_string()
        cell.set_property('cell_background', color)
        
    def initialize_widgets(self, tree):
        for widget in tree.get_objects():
            name = gtk.Buildable.get_name(widget)
            if name and not hasattr(self, name):
                setattr(self, name, widget)

    def connect_signals(self):
        self.button1.connect('clicked', self.destroy)
        self.button2.connect('clicked', self.upload)
        self.window1.connect('delete_event', self.destroy)
        self.window1.connect('key_press_event', self.on_key_press_event)
        self.button5.connect('clicked', self.change_user_account)

    def on_key_press_event(self, obj, event):
        keyname = gtk.gdk.keyval_name(event.keyval)
        if keyname == 'Delete' and len(self.model):
            selection = self.treeview1.get_selection()
            model, iter_ = selection.get_selected()
            if iter_:
                model.remove(iter_)

    def populate_dir_box(self, dirs, text=None):
        self.boxmodel1.clear()
        self.boxmodel1.append(["/"])
        for dir_ in sorted(dirs):
            self.boxmodel1.append([dir_])
        if text:
            for row in self.boxmodel1:
                if row == text:
                    self.combobox1.set_active_iter(row.iter)
                    break
        else:
            self.combobox1.set_active(0)

    def upload(self, obj=None):
        if len(self.model):
            for row in self.model:
                self.model.set_value(row.iter, 2, '')
            if self.connection.is_logged_in():
                self.button2.set_sensitive(False)
                self.stop = False
                self.uploading = True
                self.upload_to_dropbox()
                self.stop = False
                self.uploading = False
                self.button2.set_sensitive(True)
            else:
                self.label3.set_text(_('not logged!'))
                self.dropbox_login()

    def upload_to_dropbox(self):
        remote_dir = self.combobox1.get_active_text()
        fraction = 0.99 / len(self.model)
        self.progressbar1.set_fraction(0)
        self.progressbar1.show()
        while gtk.events_pending():
            gtk.main_iteration(False)
        for row in self.model:
            if self.stop:
                break
            image = self.get_source_file(row[1])
            self.progressbar1.set_text(_('Uploading %s ...') % row[1])
            self.model.set_value(row.iter, 2, 'uploading...')
            while gtk.events_pending():
                gtk.main_iteration(False)
            if not remote_dir.startswith('/'):
                remote_dir = '/' + remote_dir
            try:
                self.connection.upload_file(image, remote_dir, image)
                self.model.set_value(row.iter, 2, 'completed!')
            except:
                self.model.set_value(row.iter, 2, 'error!')
                sys.stderr.write(traceback.format_exc())
            self.progressbar1.set_fraction(self.progressbar1.get_fraction() + fraction)
            self.treeview1.scroll_to_cell(self.model.get_path(row.iter))
            while gtk.events_pending():
                gtk.main_iteration(False)
        time.sleep(1)
        self.progressbar1.hide()

    def get_source_file(self, basename):
        return self.gallery.get_real_path(self.rawpath[basename])
                           
    def refresh_treeview(self):
        self.rawpath = {}
        for item in self.gallery.selected:
            basename = os.path.basename(self.gallery.get_real_path(item))
            self.rawpath[basename] = item
            if self.image_list[0] == self.gallery.image28:
                pixbuf = self.gallery.scale_thumbnail(self.image_list[0].get_pixbuf().copy())
            else:
                pixbuf = self.gallery.scale_thumbnail(self.image_list[self.gallery.list.index(item)].get_pixbuf().copy())
            self.model.append([pixbuf, basename, None])
            while gtk.events_pending():
                gtk.main_iteration(False)
                
    def dialog(self):
        self.dialog1.show()
        if self.entry2.get_text():
            self.entry3.grab_focus()
        while gtk.events_pending():
            gtk.main_iteration(False)
        res = self.dialog1.run()
        if res in [-4, -6]:
            self.dialog1.hide()
            return None
        else:
            user = self.entry2.get_text()
            if not user:
                self.dialog()
            pwd = self.entry3.get_text()
            if not pwd:
                self.dialog()
            self.dialog1.hide()
            while gtk.events_pending():
                gtk.main_iteration(False)
            return user, pwd

    def change_user_account(self, obj):
        self.label3.set_text('')
        while gtk.events_pending():
            gtk.main_iteration(False)
        self.refresh_db_dirs()
        
    def refresh_db_dirs(self):
        self.window1.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        while gtk.events_pending():
            gtk.main_iteration(False)
        if self.dropbox_login():
            try:
                if self.connection.is_logged_in():
                    self.label3.set_text(_('Retrieving directories...'))
                    while gtk.events_pending():
                            gtk.main_iteration(False)
                    dirs = self.connection.get_dir_list('/')
                    self.populate_dir_box(dirs)
                    self.button2.set_sensitive(True)
                    self.label3.set_text(_('You are logged!'))
                else:
                    self.label3.set_text(_('Login failed!'))
                    sys.stderr.write(traceback.format_exc())
                self.window1.window.set_cursor(None)
                return True
            except:
                self.label3.set_text(_('Login failed!'))
                self.window1.window.set_cursor(None)
                return True
        self.window1.window.set_cursor(None)
        return False
                
    def dropbox_login(self):
        credentials = self.dialog()
        if credentials:
            username, pwd = credentials
        else:
            return None
        try:
            self.label3.set_text(_('authenticating...'))
            while gtk.events_pending():
                gtk.main_iteration(False)
            self.connection = DropboxConnection(username, pwd)
            self.gallery.settings['dropbox_user'] = username
            self.gallery.settings['dropbox_pwd'] = pwd
            self.window1.set_title(_('Upload into %s DropBox' % username))
            self.label3.set_text(_('You are logged!'))
        except:
            gtk.gdk.beep()
            self.label3.set_text(_('Log-in failed!'))
            sys.stderr.write(traceback.format_exc())
            return None
        return True
                     
    def main(self):
        self.window1.show()
        self.progressbar1.hide()
        self.refresh_treeview()
        self.refresh_db_dirs()
            
    def destroy(self, *obj):
        if self.uploading:
            self.stop = True
            return True
        self.window1.hide_all()
        self.gallery.unselect()
        while gtk.events_pending():
            gtk.main_iteration(False)
        self.model.clear()
        return True


